## This experiment compares the computational times
## of the 5 different methods presented in the article

library(bnlearn)
library(glasso)
# Set counters to 0
t.CL <- 0
t.EM_tree <- 0
t.Glasso <- 0
t.EM_Glasso <- 0
t.EM_tree_agg <- 0
t.init <- 0

nb.iter <- 50 # Nb of replications
n <- 200 # Nb of samples

# Load initialized data
adjmat <- as.matrix(read.table(file = "../data/tree/adjmat_small_V2.txt"))
Sigma <- as.matrix(read.table(file = "../data/tree/Sigma_small_V2.txt"))
K <- solve(Sigma)
K_O <- K[-idx, -idx]
K_m <- K_O - K[-idx,idx, drop=F]%*%K[idx,idx]%*%t(K[-idx,idx, drop=F])
idx <- indices_small[i]
Sigma1 <- Sigma[-idx,-idx]
clique0 <- which(adjmat[idx,]>0)
clique0 <- sapply(clique0, function(x) x-(x>idx))
adjmat_marg <- adjmat[-idx, -idx]
adjmat_marg[clique0, clique0] <- 1

# Sample incomplete data
X <- mvrnorm(n, mu = rep(0, nrow(Sigma1)), Sigma1)
X <- scale(X, center = TRUE, scale = TRUE)
S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))

# Initialize with hierarchical clustering
initClique <- getInitCliques(S,n)
num_clusts <- length(initClique)
clique <- initClique$cliquelist[[1]]
init <- initEM(X, cliquelist=list(clique), n=n)
Sigma0 <- init$Sigma0
K0 <- init$K0

for (i in 1:nb.iter){

  t <- Sys.time()
  CL <- inferAdjMat(S, algo_type = "CL")
  t.CL <- c(t.CL, Sys.time() - t)

  t <- Sys.time()
  EM.res <- tree.EM(S, k=num_clusts, K0, Sigma0, max.iter = 1e3)
  t.EM_tree <- c(t.EM_tree, Sys.time() - t)

  t <- Sys.time()
  MB.res <- lapply(exp(log.lambda), function(lambda) glasso(S, lambda, trace = TRUE, approx = TRUE,
                                                            penalize.diagonal = FALSE))
  t.Glasso <- c(t.Glasso, Sys.time() - t)

  t <- Sys.time()
  EM.Agr.res <- treeAgr.EM(S, k=num_clusts, K0, Sigma0, n, pi=0.5, max.iter=1e3, eps=1e-3)
  t.EM_tree_agg <- c(t.EM_tree_agg, Sys.time() - t)

  t <- Sys.time()
  initClique <- getInitCliques(S,n)
  num_clusts <- length(initClique)
  clique <- initClique$cliquelist[[1]]
  t.init <- c(t.init, Sys.time() - t)
}

tab_large_rank <- data.frame(algo = c("Chow-Liu", "EM-Chow-Liu", "Glasso", "EM-Glasso", "EM-TreeAgg"),
                             time = c(mean(t.CL[2:nb.iter])/nb.iter, mean(t.EM_tree[2:nb.iter])/nb.iter, mean(t.Glasso[2:nb.iter])/nb.iter,
                                                            NA, mean(t.EM_tree_agg[2:nb.iter])/nb.iter))

library(xtable)
xtable(tab_large_rank)
