# This experiment performs model selection
# for the flow cytometry data experiment

setwd("~/latent-tree-aggregation/to-send-smj/code/functions")
load("~/latent-tree-aggregation/to-send-smj/code/data/Raf_data/RAF_adj.RData")
load("~/latent-tree-aggregation/to-send-smj/code/data/Raf_data/Netbuilder_Obs_Noise_01.RData")
load("~/latent-tree-aggregation/to-send-smj/code/data/Raf_data/Real_Obs.RData")
source("ChowLiu.R")
source("HierarchicalClustering.R")
source("inferAdjmat.R")
source("statTests.R")
source("EM.R")
source("TreeAgregation.R")
source("RecursiveGrouping.R")
source("critere.R")
library(igraph)
library(bnlearn)

# load data
RAF_original_graph <- graph.adjacency(RAF_original_adj_undirected)
idx <- 8 # index of hidden variable
# pka: 8 , pkc: 9, mek: 2
RAF <- RAF_original_adj_undirected

# "Hide" variable idx
n <- 100
X <- as.matrix(Real_Obs_1)
X <- X[, -idx]
X <- scale(X, center=TRUE, scale=TRUE)
Sigma <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
p <- nrow(S)

nlambda <- 50 # length of regularization path for glasso
num <- 55 # number of edges in the Tree Aggregation graph
numEdges <- 1:55

# Initialization
initClique <- getInitCliques(S,n)
hclas=initClique$res$labelslist
max.K=length(hclas)
cliquelist <- initClique$cliquelist

crit=list()
EM.resm=list()
for(k in 1:2){
  cliquelist2 <- lapply(1:k, function(k) cliquelist[[k]])
  S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
  init <- initEM(X, cliquelist=cliquelist2, n=n, num_clusts=k)
  Sigma0 <- init$Sigma0
  K0 <- init$K0

  # Agregate EM ----------------------------------------------------------------------------------------------
  EM.resm[[k]] <- treeAgr.EM(S, k=k, K0, Sigma0, n, pii=0.5,eps=1e-3,max.iter=1e3)
  log_gamma=EM.resm[[k]]$gamma
  alpha <- edge.prob(log_gamma, account.prior = TRUE, q0 = 0.5)
  edges=alpha>0.5
  graph_cyto=graph.adjacency(edges)
  colors <- rep('gray',nrow(alpha))
  colors[12:nrow(alpha)] <- 'red'
  crit[[k]]=critere(X, EM.resm[[k]], pii=0.5)
}

ICL_T=sapply(crit, function(x) x$ICL_T)
ICL_ZT=sapply(crit, function(x) x$ICL_ZT)
BIC=sapply(crit, function(x) x$BIC)


## Compute criteria for number of hidden nodes = 0
K0=solve(S)
EM.Agr.res <- treeAgr.EM(S, k=0, K0, Sigma0=S, n, pii=0.5,eps=1e-2,max.iter=1)
K=EM.Agr.res$K
Sigma=EM.Agr.res$Sigma
log_gamma=EM.Agr.res$gamma
log_gamma[log_gamma>0]=0
alpha <- edge.prob(log_gamma, account.prior = TRUE, q0 = 0.5)
P=EM.Agr.res$P
diag(log_gamma)=0
n=nrow(X)
q=nrow(log_gamma)
p=ncol(X)
r=nrow(Sigma)-p
#pii=log(matrix(2/(p+r),p+r,p+r))/n
pii=log(matrix(0.5,p+r,p+r))/n
diag(pii)=0
diag(P)=0
delta=-exp(log_gamma)
diag(delta)=rowSums(exp(log_gamma))
H.T=log(det(delta[2:p,2:p]))-sum(sum(log_gamma*alpha)) # Entropy of trees
pen.r=(log(n)/n)*(p*(p+1)/2)
delta0 = -exp(pii)+diag(rowSums(exp(pii)))
Z0 = det(delta0[2:p,2:p])
mat=alpha*(pii+P)
E.c=sum(sum(mat)) - log(Z0)-(p+r)*log(2*pi)/2# Expectation of complete likelihood
logPY = E.c+H.T # Marginal log-likelihood of observed variables

ICL_T=c(logPY-H.T-pen.r,ICL_T)
BIC=c(logPY-pen.r,BIC)
ICL_ZT=c("NA",ICL_ZT)
vals=as.numeric(c(ICL_T,BIC,ICL_ZT)[!(c(ICL_T,BIC,ICL_ZT)=="NA")])
vals=vals[!vals=="NaN"]

# Plots
plot(NA, xlim=c(-0.2,4), ylim=c(min(vals), max(vals)),xlab="number of hidden nodes",ylab="")
points(0:2,ICL_T,type="b",col="purple",pch=15,cex=1)
points(0:2,BIC,type="b",col="blue",pch=19,cex=1)
points(0:2,ICL_ZT,type="b",col="green",pch=18,cex=1)
legend("topright",fill=c("purple","blue","green"),legend=c("ICL Tree", "BIC", "ICL Tree & hidden"))
