# This experiment reproduces the ROC curves presented in the article
# on synthetic data

setwd("~/latent-tree-aggregation/to-send-smj/code/functions") # Path needs to be changed to appropriate path
source("ChowLiu.R")
source("HierarchicalClustering.R")
source("inferAdjmat.R")
source("statTests.R")
source("EM.R")
source("TreeAgregation.R")
source("critere.R")
library(matrixcalc)
library(saturnin)
library(Matrix)
library(psych)
library(RColorBrewer)
library(mvtnorm)
require(matrixStats)
require(pracma)
require(cluster)
require(glasso)
library(abind)
library(gtools)


cols <- c(brewer.pal(9, "Set1"), brewer.pal(8, "Set2")) # color settings

graph.type <- c("tree", "erdos") # Experiment for two types of graphs
algos <- c("CL", "MB", "EM-CL", "EM-MB", "EM-agregation") # Compare 5 methods
indices_small=c(11,9) # indices of variables that will be hidden


n <- 200 # nb of samples
nb.iter <- 50
nlambda <- 20
rank = NULL
for (i in 1:2){ # 1 tree 2 erdos
  # Load ground truth: true adjacency matrix and covariance matrix
  adjmat <- as.matrix(read.table(file = paste("../data/", graph.type[i],
                                               "/adjmat_small_V2.txt", sep = "")))
  Sigma <- as.matrix(read.table(file = paste("../data/", graph.type[i],
                                              "/Sigma_small_V2.txt", sep = "")))
  K <- solve(Sigma)
  K_O <- K[-idx, -idx]
  K_m <- K_O - K[-idx,idx, drop=F]%*%K[idx,idx]%*%t(K[-idx,idx, drop=F])
  idx <- indices_small[i]
  Sigma1 <- Sigma[-idx,-idx]
  clique0 <- which(adjmat[idx,]>0)
  clique0 <- sapply(clique0, function(x) x-(x>idx))
  adjmat_marg <- adjmat[-idx, -idx]
  adjmat_marg[clique0, clique0] <- 1


  # vector containing the number of edges wanted for each try of EM with tree agregation
  numEdges <- floor(c(seq(1,100, length.out=20), seq(110, 500, length.out=20), seq(550, 1150,length.out=20)))
  numEdges=numEdges[1:36]
  num <- length(numEdges)

  pow.MB_marg <- rep(0, nlambda)
  FDR.MB_marg <- rep(0, nlambda)

  pow.EM.Glasso_marg <- rep(0, nlambda)
  FDR.EM.Glasso_marg <- rep(0, nlambda)

  pow.EM.Agr_marg <- rep(0, num)
  FDR.EM.Agr_marg <- rep(0, num)

  pow.CL_marg <- 0
  FDR.CL_marg <- 0

  pow.EM_marg <- 0
  FDR.EM_marg <- 0

  pow.RG_marg <- 0
  FDR.RG_marg <- 0


  pow.EM.Glasso <- rep(0, nlambda)
  FDR.EM.Glasso <- rep(0, nlambda)

  pow.EM.Agr <- rep(0, num)
  FDR.EM.Agr <- rep(0, num)

  pow.EM <- 0
  FDR.EM <- 0
  count=0
  for (iter in 1:nb.iter) {
    count=count+1
    X <- mvrnorm(n, mu = rep(0, nrow(Sigma1)), Sigma1)
    X <- scale(X, center = TRUE, scale = TRUE)
    S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
    # Initialize with hierarchical clustering
    initClique <- getInitCliques(S,n)
    num_clusts <- length(initClique)
    clique <- initClique$cliquelist[[1]]
    init <- initEM(X, cliquelist=list(clique), n=n)
    Sigma0 <- init$Sigma0
    K0 <- init$K0
    num_clusts <- 1

    CL <- inferAdjMat(S, algo_type = "CL")
    CL <- CL$adjmat_marg

    ## Comparison with marginal graph
    pow.CL_marg <- pow.CL_marg + (1/nb.iter) * power(adjmat_marg, CL)
    FDR.CL_marg <- FDR.CL_marg + (1/nb.iter) * FDR(adjmat_marg, CL)

    # Tree EM -----------------------------------------------------------------
    EM.res <- tree.EM(S, k=num_clusts, K0, Sigma0, max.iter = 1e3)
    EM.Tree_adjmat <- EM.res$adjmat
    EM.K <- EM.res$K
    EM.Tree_adjmat_cond <- EM.Tree_adjmat[1:p, 1:p]
    EM.K_marg <- EM.K[1:p, 1:p] - EM.K[1:p, (p+1):(p+num_clusts)] %*% solve(EM.K[(p+1):(p+num_clusts), (p+1):(p+num_clusts)]) %*% EM.K[(p+1):(p+num_clusts),                                                                                                                                          1:p]
    EM.Tree_adjmat_marg <- (EM.K_marg != 0) * 1
    diag(EM.Tree_adjmat_marg) <- 0

    ## Comparison with marginal graph
    pow.EM_marg <- pow.EM_marg + (1/nb.iter) * power(adjmat_marg, EM.Tree_adjmat_marg)
    FDR.EM_marg <- FDR.EM_marg + (1/nb.iter) * FDR(adjmat_marg, EM.Tree_adjmat_marg)
    ## Comparison with full graph
    pow.EM <- pow.EM + (1/nb.iter) * power(adjmat[-idx,-idx], EM.Tree_adjmat_cond)
    FDR.EM <- FDR.EM + (1/nb.iter) * FDR(adjmat[-idx,-idx], EM.Tree_adjmat_cond)

    # Glasso -------------------------------------------------------------------------------------------------
    log.lambda.min <- -5
    lambda.max <- get.lambda.l1(S)
    log.lambda.max <- log(lambda.max)
    log.lambda <- seq(log.lambda.min, log.lambda.max, length.out = nlambda)

    MB.res <- lapply(exp(log.lambda), function(lambda) glasso(S, lambda, trace = TRUE, approx = FALSE,
                                                              penalize.diagonal = FALSE))
    MB.Ks <- lapply(MB.res, function(res) res$wi)
    mat <- matrix(1, p , p )
    diag(mat) <- 0
    MB.adjmats <- lapply(MB.Ks, function(K) (abs(K) > 0) * 1 * mat)

    ## Comparison with marginal graph
    pow.MB_marg <- pow.MB_marg + (1/nb.iter) * sapply(MB.adjmats, function(x) power(adjmat_marg,x))
    FDR.MB_marg <- FDR.MB_marg + (1/nb.iter) * sapply(MB.adjmats, function(x) FDR(adjmat_marg,x))

    # Convex EM ----------------------------------------------------------------------------------------------
    nlambda <- 20
    log.lambda.min <- -1
    lambda.max <- get.lambda.l1(Sigma0)
    log.lambda.max <- log(lambda.max)
    log.lambda <- seq(log.lambda.min, log.lambda.max, length.out = nlambda)

    EM.Glasso.res <- lapply(exp(log.lambda), function(lambda) convex.EM(S, k=num_clusts, lambda, K0, Sigma0, max.iter = 10, eps=1e-3))
    EM.Glasso_adjmats <- lapply(EM.Glasso.res, function(res) res$adjmat)
    EM.Glasso.Ks <- lapply(EM.Glasso.res, function(res) res$K)
    EM.Glasso.K_cond <- lapply(EM.Glasso.Ks, function(K) K[1:p, 1:p])
    EM.Glasso.K_inter <- lapply(EM.Glasso.Ks, function(K) K[1:p, (p+1):(p+num_clusts)])
    EM.Glasso.K_marg <- lapply(1:nlambda, function(x) EM.Glasso.K_cond[[x]] + EM.Glasso.K_inter[[x]] %*%
                                 t(EM.Glasso.K_inter[[x]]))

    EM.Glasso_adjmat_marg <- lapply(EM.Glasso.K_marg, function(K) (abs(K) > 0) * 1 * mat)

    ## Comparison with marginal graph
    pow.EM.Glasso_marg <- pow.EM.Glasso_marg + (1/nb.iter) * sapply(EM.Glasso_adjmat_marg, function(x) power(adjmat_marg,x))
    FDR.EM.Glasso_marg <- FDR.EM.Glasso_marg + (1/nb.iter) * sapply(EM.Glasso_adjmat_marg, function(x) FDR(adjmat_marg,x))
    ## Comparison with full graph
    pow.EM.Glasso <- pow.EM.Glasso + (1/nb.iter) * sapply(EM.Glasso_adjmats, function(x) power(adjmat[c(setdiff(1:(p+1),idx),idx),c(setdiff(1:(p+1),idx),idx)], x))
    FDR.EM.Glasso <- FDR.EM.Glasso + (1/nb.iter) * sapply(EM.Glasso_adjmats, function(x) FDR(adjmat[c(setdiff(1:(p+1),idx),idx),c(setdiff(1:(p+1),idx),idx)], x))

    # Agregate EM ----------------------------------------------------------------------------------------------
    EM.Agr.res <- treeAgr.EM(S, k=num_clusts, K0, Sigma0, n, pi=0.5, max.iter=1e3, eps=1e-3)
    alpha <- EM.Agr.res$alpha
    lik <- EM.Agr.res$likelihood
    ordered_couples <- EM.Agr.res$ordered_couples
    EM.Agr_adjmats <- lapply(numEdges, function(m) constructGraphFromAlpha(abs(alpha), m, ordered_couples))
    mat <- matrix(1, p, p)
    diag(mat) <- 0
    EM.Agr_adjmat_marg <- lapply(EM.Agr_adjmats, function(a) (a[1:p,1:p]+
                                                                a[1:p, (p+1):(p+num_clusts)]%*%t(a[(p+1):(p+num_clusts), 1:p]))*mat)
    EM.Agr_adjmat_marg <- lapply(EM.Agr_adjmat_marg, function(a) 1*(a>0))
    EM.Agr.Ks <- lapply(EM.Agr_adjmats, function(x) x*EM.Agr.res$K)

    ## Comparison with marginal graph
    pow.EM.Agr_marg <- pow.EM.Agr_marg + (1/nb.iter) * sapply(EM.Agr_adjmat_marg, function(x) power(adjmat_marg, x))
    FDR.EM.Agr_marg <- FDR.EM.Agr_marg + (1/nb.iter) * sapply(EM.Agr_adjmat_marg, function(x) FDR(adjmat_marg, x))

    ## Comparison with full graph
    pow.EM.Agr <- pow.EM.Agr + (1/nb.iter) * sapply(EM.Agr_adjmats, function(x) power(adjmat[c(setdiff(1:(p+1),idx),idx),c(setdiff(1:(p+1),idx),idx)], x))
    FDR.EM.Agr <- FDR.EM.Agr + (1/nb.iter) * sapply(EM.Agr_adjmats, function(x) FDR(adjmat[c(setdiff(1:(p+1),idx),idx),c(setdiff(1:(p+1),idx),idx)], x))


    # rank of initial clique
    y <- alpha[p, clique]
    z <- alpha[1:(p-1), p]
    x <- alpha[upper.tri(alpha)]
    r = p-rank(z)
    rank = c(rank, mean(sapply(z, function(yy) r[which(z==yy)])))

  }

  library(ggplot2)
  ## PLOT ROC curves
  df_pr <- rbind(data.frame(pow=pow.EM.Agr_marg, fdr=FDR.EM.Agr_marg),
                 data.frame(pow=pow.EM.Glasso_marg, fdr=FDR.EM.Glasso_marg),
                 data.frame(pow=pow.MB_marg, fdr=FDR.MB_marg),
                 data.frame(pow=pow.CL_marg, fdr=FDR.CL_marg),
                 data.frame(pow=pow.EM_marg, fdr=FDR.EM_marg),
                 data.frame(pow=pow.RG_marg, fdr=FDR.RG_marg),
                 data.frame(pow=1, fdr=1))
  df_pr[,"Method"] <- factor(c(rep("EM Tree Agregation",length(pow.EM.Agr_marg)),
                               rep("EM-Glasso",length(pow.EM.Glasso_marg)),
                               rep("Glasso", length(pow.MB_marg)),
                               "Chow-Liu", "EM-Chow-Liu", "Recursive Grouping","Limit-Point"), levels=c("EM Tree Agregation", "Chow-Liu", "Glasso", "EM-Glasso", "EM-Chow-Liu", "Recursive Grouping","Limit-Point"))


  df3_pr <- rbind(data.frame(pow=pow.EM.Agr, fdr=FDR.EM.Agr),
                  data.frame(pow=pow.EM.Glasso, fdr=FDR.EM.Glasso),
                  data.frame(pow=pow.EM, fdr=FDR.EM))
  df3_pr[,"Method"] <- factor(c(rep("EM Tree Agregation",length(pow.EM.Agr)),
                                rep("EM-Glasso",length(pow.EM.Glasso)),
                                "EM-Chow-Liu"), levels=c("EM Tree Agregation", "EM-Glasso", "EM-Chow-Liu"))


  ggplot(df_pr)+aes(x=fdr, y=pow, colour=Method)+geom_point(aes(shape=Method, size=Method))+scale_color_manual(values=c("green", "black", "pink","red","blue","purple","black"))+scale_shape_manual(values=c(43, 17, 20, 42, 15, 18, 19))+scale_size_manual(values=c(5, 4, 3, 6, 4, 5, 5))+theme(axis.title.x=element_blank(),
                                                                                                                                                                                                                                                                                                 axis.title.y=element_blank(),legend.position="none")

  ggplot(df3_pr)+aes(x=fdr, y=pow, colour=Method)+geom_point(aes(shape=Method, size=Method))+scale_color_manual(values=c("green", "black", "pink","red","blue","purple", "black"))+scale_shape_manual(values=c(43, 17, 20, 42, 15, 18, 19))+scale_size_manual(values=c(5, 4, 3, 6, 4, 5, 5))+theme(axis.title.x=element_blank(),
                                                                                                                                                                                                                                                                                                   axis.title.y=element_blank(),legend.position="none")

}








