## This file reproduces the model selection results
## when 0 variable is hidden, i.e. the complete data
## is observed

setwd("~/latent-tree-aggregation/to-send-smj/code/functions")

###################################### SOURCE FILES ######################################
source("ChowLiu.R")
source("HierarchicalClustering.R")
source("inferAdjmat.R")
source("EM.R")
source("TreeAgregation.R")
source("critere.R")
source('statTests.R')
source('MeinshausenBuhlmann.R')
source("buildGraph.R")
source("sampleGraph.R")

####################################### LIBRARIES #########################################
require(matrixStats)
require(pracma)
require(cluster)
require(igraph)
require(bnlearn)

require(Hmisc)
require(RColorBrewer)
require(dendextend)
require(igraph)
require(mvtnorm)

######################################## PARAMETERS ########################################


cols <- c(brewer.pal(9, "Set1"), brewer.pal(8, "Set2"))
graph_types <- c("tree", "erdos", "doublestar")
n          <- 30
nb_hidden  <- 1
prop.positive.cor <- 1

nb.iter <- 50
nlambda <- 20
eps1 <- 1
eps2 <- 0.1
crit_list <- list()
graph_type <- "tree"
######################################### LOAD DATA ###########################################


file_adjmat = paste('../simu-V2/',graph_type,'/adjmat_',graph_type,'.Rdata',
                    sep = '')
load(file_adjmat)
degrees <- rowSums(adjmat)
idx <- which.max(degrees)[1]
p0      <- nrow(adjmat)
######################################" GENERATE PRECISION MATRIX ######################################

params <- runif(p0*p0, min=0.8, max=1) * (2*rbinom(p0*p0,size=1,prob=1-prop.positive.cor)-1)
params <- matrix(params, nrow=p0, byrow=TRUE)
params <- (params+t(params))/2

params <- params*adjmat
params[-idx,-idx] <- eps1*params[-idx,-idx]
params[idx,-idx] <- eps2*params[idx,-idx]
params[-idx,idx] <- eps2*params[-idx,idx]
diag(params) <- 1.25 * rowSums(abs(params))

K  <- params
Sigma  <- solve(K)
Sigma1            <- Sigma[-idx, -idx]
adjmat_cond       <- (K[-idx, -idx]!=0)*1
diag(adjmat_cond) <- 0
K0_marg           <- K[-idx, -idx, drop = FALSE]-K[-idx, idx,
                                                   drop = FALSE]%*%solve(K[idx, idx,
                                                                           drop = FALSE])%*%K[idx,-idx, drop = FALSE]
adjmat_marg       <- (K0_marg!=0)*1
diag(adjmat_marg) <- 0

algos <- c("CL", "MB", "EM-CL", "EM-MB", "EM-agregation")

X <- mvrnorm(n, mu = rep(0, nrow(Sigma1)), Sigma1)
X <- scale(X, center=TRUE, scale=TRUE)
S <- 1/n * t(X) %*% X
p <- nrow(S)

# Initialize with hierarchical clustering
initClique <- getInitCliques(S,n)
num_clusts <- length(initClique)

clique <- initClique$cliquelist[[1]]
count=0
BIC <- rep(0, nb.iter)
ICL_T <- rep(0, nb.iter)
for (iter in 1:nb.iter) {
  count=count+1
  X <- mvrnorm(n, mu = rep(0, nrow(Sigma1)), Sigma1)
  X <- scale(X, center = TRUE, scale = TRUE)
  S <- 1/n * t(X) %*% X
  p <- nrow(S)
  init <- initEM(X, cliquelist=list(clique), n=n)
  Sigma0 <- init$Sigma0
  K0 <- init$K0
  num_clusts <- 1
  EM.Agr.res <- treeAgr.EM(S, k=0, K0, Sigma0=S, n, pii=0.5,eps=1e-2,max.iter=1)
  K=EM.Agr.res$K
  Sigma=EM.Agr.res$Sigma
  log_gamma=EM.Agr.res$gamma
  log_gamma[log_gamma>0]=0
  #alpha=EM.Agr.res$alpha
  alpha <- edge.prob(log_gamma, account.prior = TRUE, q0 = 0.5)
  P=EM.Agr.res$P
  diag(log_gamma)=0
  n=nrow(X)
  q=nrow(log_gamma)
  p=ncol(X)
  r=nrow(Sigma)-p
  #pii=log(matrix(2/(p+r),p+r,p+r))/n
  pii=log(matrix(0.5,p+r,p+r))
  diag(pii)=0
  diag(P)=0
  delta=-exp(log_gamma)
  diag(delta)=rowSums(exp(log_gamma))
  H.T=log(det(delta[2:p,2:p]))-sum(sum(log_gamma*alpha)) # Entropy of trees
  pen.r=(log(n))*(p*(p+1)/2)
  delta0 = -exp(pii)+diag(rowSums(exp(pii)))
  Z0 = det(delta0[2:p,2:p])
  mat=alpha*(pii+P)
  E.c=sum(sum(mat)) - log(Z0)-(p+r)*log(2*pi)/2# Expectation of complete likelihood
  logPY = E.c+H.T # Marginal log-likelihood of observed variables
  BIC[count] <- logPY-pen.r
  ICL_T[count] <- logPY-H.T-pen.r
}
######## PLOT CRITERE
library(ggplot2)
BIC_matrix <- do.call(rbind, BIC_list)
#BIC_matrix <- cbind(BIC, BIC_matrix)
BIC_matrix <- BIC_matrix + pen_add_mat



BIC_ggplot <- c(BIC_matrix[,1], BIC_matrix[,2], BIC_matrix[,3], BIC_matrix[,4])
BIC_ggplot <- cbind(BIC_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
BIC_ggplot <- data.frame(BIC_ggplot)
colnames(BIC_ggplot) <- c("critere.val", "nbr.hidden")
BIC_ggplot$nbr.hidden <- as.factor(BIC_ggplot$nbr.hidden)

ggplot(BIC_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()

ICL_T_matrix <- do.call(rbind, ICL_T_list)
#ICL_T_matrix <- cbind(ICL_T_matrix, ICL_T )
ICL_T_matrix <- ICL_T_matrix + pen_add_mat

ICL_T_ggplot <- c(ICL_T_matrix[,1], ICL_T_matrix[,2], ICL_T_matrix[,3],ICL_T_matrix[,4])
ICL_T_ggplot <- cbind(ICL_T_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
ICL_T_ggplot <- data.frame(ICL_T_ggplot)
colnames(ICL_T_ggplot) <- c("critere.val", "nbr.hidden")
ICL_T_ggplot$nbr.hidden <- as.factor(ICL_T_ggplot$nbr.hidden)

ggplot(ICL_T_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()

ICL_ZT_list <- lapply(ICL_ZT_list, as.numeric)
ICL_ZT_matrix <- do.call(rbind, ICL_ZT_list) + pen_add_mat

ICL_ZT_ggplot <- c(as.numeric(ICL_ZT_matrix[,1]), as.numeric(ICL_ZT_matrix[,2]), as.numeric(ICL_ZT_matrix[,3]), as.numeric(ICL_ZT_matrix[,4]))
ICL_ZT_ggplot <- cbind(ICL_ZT_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
ICL_ZT_ggplot <- data.frame(ICL_ZT_ggplot)
colnames(ICL_ZT_ggplot) <- c("critere.val", "nbr.hidden")
ICL_ZT_ggplot$nbr.hidden <- as.factor(ICL_ZT_ggplot$nbr.hidden)

ggplot(ICL_ZT_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()


critere_ggplot <- rbind.data.frame(BIC_ggplot, ICL_T_ggplot, ICL_ZT_ggplot)
critere_ggplot$critere.type <- c(rep("BIC", 200), rep("ICL_Tree", 200), rep("ICL_Tree_Hid", 200))
critere_ggplot$critere.type <- as.factor(critere_ggplot$critere.type)

ggplot(critere_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot() + facet_wrap(~critere.type, nrow = 3) + theme(axis.title.x=element_blank(),
                                                                                                                    axis.title.y=element_blank(),
                                                                                                                    panel.background = element_rect(fill="white"))


