# This file performs model selection for two types of graphs (tree and erdos),
# in the case where 1 variable is unobserved

setwd("~/latent-tree-aggregation/to-send-smj/code/functions")
source("ChowLiu.R")
source("HierarchicalClustering.R")
source("inferAdjmat.R")
source("statTests.R")
source("EM.R")
source("TreeAgregation.R")
source("RecursiveGrouping.R")
source("critere.R")
# Set parameters
graph.type <- c("tree", "erdos")
n <- 200

for (i in 1:2){
  file_adjmat = paste('../simu-V2/',graph_type,'/adjmat_',graph_type,'.Rdata',
                      sep = '')
  load(file_adjmat)
  degrees <- rowSums(adjmat)
  p0      <- nrow(adjmat)

  graph <- graph.adjacency(adjmat)
  plot(graph)
  initClique <- getInitCliques(Sigma,n)
  num_clusts <- length(initClique)
  cliquelist <- initClique$cliquelist

  library(mvtnorm)
  scan(file=paste("../Data/", graph.type[i],
                  "/indices.txt", sep = ""))->idx
  Sigma1 <- Sigma[-idx,-idx]
  X <- mvrnorm(n, mu = rep(0, nrow(Sigma)), Sigma)
  X <- scale(X, center=TRUE, scale=TRUE)
  Sigma <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
  X <- X[,setdiff(1:ncol(X),idx)]
  S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
  p <- nrow(S)
  clique0 <- which(adjmat[idx,]>0)
  clique0 <- sapply(clique0, function(x) x-(x>idx))
  adjmat_marg <- adjmat[-idx, -idx]
  adjmat_marg[clique0, clique0] <- 1

  # ---------------------------------------------------------------------------------------------------------
  BIC=rep(0,10)
  ICL_T=rep(0,10)
  ICL_ZT=rep(0,10)
  res=list()
  for(k in 1:10){
    cliquelist2 <- lapply(1:k, function(k) cliquelist[[k]])
    S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
    init <- initEM(X, cliquelist=cliquelist2, n=n, num_clusts=k)
    Sigma0 <- init$Sigma0
    K0 <- init$K0

    # Agregate EM ----------------------------------------------------------------------------------------------
    EM.Agr.res <- treeAgr.EM(S, k=k, K0, Sigma0, n, pii=0.1, max.iter=50, eps=1e-3)
    res[[k]]=EM.Agr.res
    alpha <- EM.Agr.res$alpha
    lik <- EM.Agr.res$likelihood
    ordered_couples <- EM.Agr.res$ordered_couples
    mat <- matrix(1, p, p)
    diag(mat) <- 0
    crit=critere(X, EM.Agr.res)
    BIC[k]=crit$BIC
    ICL_ZT[k]=crit$ICL_ZT
    ICL_T[k]=crit$ICL_T
  }
  crit=rbind(BIC,ICL_T,ICL_ZT)
  K=solve(S)
  crit=c(-log(det(K))-matrix.trace(K%*%S),crit)
  df=data.frame(critere=crit, k=0:10)
  asinh_trans <- function(){
    trans_new(name = 'asinh', transform = function(x) asinh(x),
              inverse = function(x) sinh(x))
  }
  library(ggplot2)
  library(scales)

  ggplot(df, aes(x=k,y=critere))+geom_point()+scale_y_continuous(trans = 'asinh', breaks=c(-100, 10e27, -10e16,-10e17))+theme_bw()

  k=which.max(crit)-1
  cliquelist2 <- lapply(1:k, function(k) cliquelist[[k]])
  S <- 1/n * t(X - repmat(colMeans(X), n, 1)) %*% (X - repmat(colMeans(X), n, 1))
  init <- initEM(X, cliquelist=cliquelist2, n=n, num_clusts=k)
  Sigma0 <- init$Sigma0
  K0 <- init$K0
  EM.Agr.res=res[[which.max(crit)]]
  alpha=EM.Agr.res$alpha
  edges=alpha>0.5
  graph=graph.adjacency(edges)
  colors <- rep('gray',nrow(alpha))
  colors[(nrow(X)+1):nrow(alpha)] <- 'red'

}




######## PLOT CRITERE
library(ggplot2)
BIC_matrix <- do.call(rbind, BIC_list)
#BIC_matrix <- cbind(BIC, BIC_matrix)
BIC_matrix <- BIC_matrix + pen_add_mat



BIC_ggplot <- c(BIC_matrix[,1], BIC_matrix[,2], BIC_matrix[,3], BIC_matrix[,4])
BIC_ggplot <- cbind(BIC_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
BIC_ggplot <- data.frame(BIC_ggplot)
colnames(BIC_ggplot) <- c("critere.val", "nbr.hidden")
BIC_ggplot$nbr.hidden <- as.factor(BIC_ggplot$nbr.hidden)

ggplot(BIC_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()

ICL_T_matrix <- do.call(rbind, ICL_T_list)
#ICL_T_matrix <- cbind(ICL_T_matrix, ICL_T )
ICL_T_matrix <- ICL_T_matrix + pen_add_mat

ICL_T_ggplot <- c(ICL_T_matrix[,1], ICL_T_matrix[,2], ICL_T_matrix[,3],ICL_T_matrix[,4])
ICL_T_ggplot <- cbind(ICL_T_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
ICL_T_ggplot <- data.frame(ICL_T_ggplot)
colnames(ICL_T_ggplot) <- c("critere.val", "nbr.hidden")
ICL_T_ggplot$nbr.hidden <- as.factor(ICL_T_ggplot$nbr.hidden)

ggplot(ICL_T_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()

ICL_ZT_list <- lapply(ICL_ZT_list, as.numeric)
ICL_ZT_matrix <- do.call(rbind, ICL_ZT_list) + pen_add_mat

ICL_ZT_ggplot <- c(as.numeric(ICL_ZT_matrix[,1]), as.numeric(ICL_ZT_matrix[,2]), as.numeric(ICL_ZT_matrix[,3]), as.numeric(ICL_ZT_matrix[,4]))
ICL_ZT_ggplot <- cbind(ICL_ZT_ggplot, c(rep(0, 50), rep(1, 50), rep(2, 50), rep(3, 50)))
ICL_ZT_ggplot <- data.frame(ICL_ZT_ggplot)
colnames(ICL_ZT_ggplot) <- c("critere.val", "nbr.hidden")
ICL_ZT_ggplot$nbr.hidden <- as.factor(ICL_ZT_ggplot$nbr.hidden)

ggplot(ICL_ZT_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot()


critere_ggplot <- rbind.data.frame(BIC_ggplot, ICL_T_ggplot, ICL_ZT_ggplot)
critere_ggplot$critere.type <- c(rep("BIC", 200), rep("ICL_Tree", 200), rep("ICL_Tree_Hid", 200))
critere_ggplot$critere.type <- as.factor(critere_ggplot$critere.type)

ggplot(critere_ggplot, aes(nbr.hidden, critere.val)) + geom_boxplot() + facet_wrap(~critere.type, nrow = 3) + theme(axis.title.x=element_blank(),
                                                                                                                    axis.title.y=element_blank(),
                                                                                                                    panel.background = element_rect(fill="white"))


